@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -23,11 +23,8 @@
 
 #include <osipparser2/osip_const.h>
 
-#include <time.h>
-
-#ifdef WIN32
-#include <Winsock.h>
-#endif
+/* Time-related functions and data types */
+#include <osip2/osip_time.h>
 
 #ifdef __sun
 #include <sys/types.h>
@@ -40,248 +37,11 @@
  * @file osip.h
  * @brief oSIP fsm Routines
  *
- * <H2>Introduction.</H2>
- * fsm stands for 'finite state machine'. The possible STATE of the state
- * machines are defined in the enum state. In oSIP, you can actually find
- * 4 different state machines. Those state machines definitions are directly
- * related to the definitions of transactions from the SIP specifications.
- * (See section: 17.1.1,  17.1.2,  17.2.1,  17.2.2). In the 4 drawings shown
- * in those sections, you'll find the possible STATES and the possible EVENTS
- * (osip_event_t) that can occur. EVENTS can be either TIMEOUT events and
- * SIP message (incoming and outgoing) events.
- * <H2>Why 4 finite state machines.</H2>
- * <BR>SIP has two different kind of transaction: INVITE and NON-INVITE ones.
- * Also, a SIP User Agent can act as a server and as a client. This simply
- * leads to 4 transactions state machines.
- * <BR>
- * <H2>Step 1: oSIP initialisation</H2>
- * <BR>To use oSIP, a program MUST first initialise internal elements in the
- * stack. The initialisation is shown below:
- * <pre><code>
- * osip_t *osip;
- * // allocate a global osip element.
- * if (0!=osip_init(&osip))
- *   return -1;
- *
- * // the next step is the initialisation of the callbacks used by the
- * // oSIP stack to announce events (when a transition occurs in the fsm)
- *
- * // This callback is somewhat special and is used by oSIP to inform
- * // the application that a message has to be sent. The message is
- * // sent by your application! oSIP has no ways to send it alone.
- * // Also, the method you supply will be called with default values where
- * // you should send the SIP message. You are not mandated to send the
- * // SIP message by using those default values.
- * // the callback MUST return 0 on success, 1 on ECONNREFUSED, -1 on error.
- *  osip_set_cb_send_message(osip, &application_cb_snd_message);
- *
- * // here is the long list of callback that you can register. Some
- * // of this callbacks are very useless (announcing a retransmission,
- * // or announcing that you have sent a SIP message which you may already
- * // know...).
- *
- * // those callbacks are mandatory. They are called when oSIP has decided
- * // that this transaction MUST no longer be handled by oSIP. (This is
- * // called in both successful or error cases scenario)
-  osip_set_kill_transaction_callback(osip ,OSIP_ICT_KILL_TRANSACTION,
-                                 &cb_ict_kill_transaction);
-  osip_set_kill_transaction_callback(osip ,OSIP_NIST_KILL_TRANSACTION,
-                                 &cb_ist_kill_transaction);
-  osip_set_kill_transaction_callback(osip ,OSIP_NICT_KILL_TRANSACTION,
-                                 &cb_nict_kill_transaction);
-  osip_set_kill_transaction_callback(osip ,OSIP_NIST_KILL_TRANSACTION,
-                                 &cb_nist_kill_transaction);
- *
- * // those callbacks are optional. The purpose is to announce retransmissions
- * // of SIP message decided by the oSIP stack. (They can be used for statistics?)
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_2XX_RECEIVED_AGAIN,
-                        &cb_rcvresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_3456XX_RECEIVED_AGAIN,
-                        &cb_rcvresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_ICT_INVITE_SENT_AGAIN,
-                        &cb_sndreq_retransmission);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_2XX_SENT_AGAIN,
-                        &cb_sndresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_3456XX_SENT_AGAIN,
-                        &cb_sndresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_IST_INVITE_RECEIVED_AGAIN,
-                        &cb_rcvreq_retransmission);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_2XX_RECEIVED_AGAIN,
-                        &cb_rcvresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_3456XX_RECEIVED_AGAIN,
-                        &cb_rcvresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_NICT_REQUEST_SENT_AGAIN,
-                        &cb_sndreq_retransmission);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_2XX_SENT_AGAIN,
-                        &cb_sndresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_3456XX_SENT_AGAIN,
-                        &cb_sndresp_retransmission);
-  osip_set_message_callback(osip ,OSIP_NIST_REQUEST_RECEIVED_AGAIN,
-                        &cb_rcvreq_retransmission);
- *
- * // those callbacks are mandatory. They are used to announce network related
- * // errors (the return code of the network callback if it was not 0)
-  osip_set_transport_error_callback(osip ,OSIP_ICT_TRANSPORT_ERROR,
-                                    &cb_transport_error);
-  osip_set_transport_error_callback(osip ,OSIP_IST_TRANSPORT_ERROR,
-                                    &cb_transport_error);
-  osip_set_transport_error_callback(osip ,OSIP_NICT_TRANSPORT_ERROR,
-                                    &cb_transport_error);
-  osip_set_transport_error_callback(osip ,OSIP_NIST_TRANSPORT_ERROR,
-                                    &cb_transport_error);
- *  
- * // those callbacks are optional. They are used to announce the initial
- * // request sent for a newly created transaction.
-  osip_set_message_callback(osip ,OSIP_ICT_INVITE_SENT,     &cb_sndinvite);
-  osip_set_message_callback(osip ,OSIP_ICT_ACK_SENT,        &cb_sndack);
-  osip_set_message_callback(osip ,OSIP_NICT_REGISTER_SENT,  &cb_sndregister);
-  osip_set_message_callback(osip ,OSIP_NICT_BYE_SENT,       &cb_sndbye);
-  osip_set_message_callback(osip ,OSIP_NICT_CANCEL_SENT,    &cb_sndcancel);
-  osip_set_message_callback(osip ,OSIP_NICT_INFO_SENT,      &cb_sndinfo);
-  osip_set_message_callback(osip ,OSIP_NICT_OPTIONS_SENT,   &cb_sndoptions);
-  osip_set_message_callback(osip ,OSIP_NICT_SUBSCRIBE_SENT, &cb_sndsubscribe);
-  osip_set_message_callback(osip ,OSIP_NICT_NOTIFY_SENT,    &cb_sndnotify);
-  osip_set_message_callback(osip ,OSIP_NICT_UNKNOWN_REQUEST_SENT, &cb_sndunkrequest);
- *
- * // those callbacks are mandatory. They are used to announce the initial
- * // response received for a transaction. (for SIP response between 100 and 199,
- * // all responses are announced because this is not a retransmission case)
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_1XX_RECEIVED, &cb_rcv1xx);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_2XX_RECEIVED, &cb_rcv2xx);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_3XX_RECEIVED, &cb_rcv3xx);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_4XX_RECEIVED, &cb_rcv4xx);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_5XX_RECEIVED, &cb_rcv5xx);
-  osip_set_message_callback(osip ,OSIP_ICT_STATUS_6XX_RECEIVED, &cb_rcv6xx);
- *  
- * // those callbacks are optional. They are used to announce the initial
- * // response sent for a transaction. (for SIP response between 100 and 199,
- * // all responses are announced because this is not a retransmission case)
-  
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_1XX_SENT, &cb_snd1xx);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_2XX_SENT, &cb_snd2xx);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_3XX_SENT, &cb_snd3xx);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_4XX_SENT, &cb_snd4xx);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_5XX_SENT, &cb_snd5xx);
-  osip_set_message_callback(osip ,OSIP_IST_STATUS_6XX_SENT, &cb_snd6xx);
- *
- * // those callbacks are mandatory. They are used to announce the initial
- * // response received for a transaction. (for SIP response between 100 and 199,
- * // all responses are announced because this is not a retransmission case)  
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_1XX_RECEIVED, &cb_rcv1xx);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_2XX_RECEIVED, &cb_rcv2xx);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_3XX_RECEIVED, &cb_rcv3xx);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_4XX_RECEIVED, &cb_rcv4xx);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_5XX_RECEIVED, &cb_rcv5xx);
-  osip_set_message_callback(osip ,OSIP_NICT_STATUS_6XX_RECEIVED, &cb_rcv6xx);
- *
- * // those callbacks are optional. They are used to announce the initial
- * // response sent for a transaction. (for SIP response between 100 and 199,
- * // all responses are announced because this is not a retransmission case)
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_1XX_SENT, &cb_snd1xx);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_2XX_SENT, &cb_snd2xx);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_3XX_SENT, &cb_snd3xx);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_4XX_SENT, &cb_snd4xx);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_5XX_SENT, &cb_snd5xx);
-  osip_set_message_callback(osip ,OSIP_NIST_STATUS_6XX_SENT, &cb_snd6xx);
- *
- * // those callbacks are mandatory. They are used to announce the initial
- * // request received for a transaction. It is not useless to notice that
- * // a special behaviour exist for the 200 OK and the ACK in the case of
- * // a successful INVITE transaction. This will be discussed later.
-  osip_set_message_callback(osip ,OSIP_IST_INVITE_RECEIVED,     &cb_rcvinvite);
-  osip_set_message_callback(osip ,OSIP_IST_ACK_RECEIVED,        &cb_rcvack);
-  osip_set_message_callback(osip ,OSIP_IST_ACK_RECEIVED_AGAIN,  &cb_rcvack2);
-  osip_set_message_callback(osip ,OSIP_NIST_REGISTER_RECEIVED,  &cb_rcvregister);
-  osip_set_message_callback(osip ,OSIP_NIST_BYE_RECEIVED,       &cb_rcvbye);
-  osip_set_message_callback(osip ,OSIP_NIST_CANCEL_RECEIVED,    &cb_rcvcancel);
-  osip_set_message_callback(osip ,OSIP_NIST_INFO_RECEIVED,      &cb_rcvinfo);
-  osip_set_message_callback(osip ,OSIP_NIST_OPTIONS_RECEIVED,   &cb_rcvoptions);
-  osip_set_message_callback(osip ,OSIP_NIST_SUBSCRIBE_RECEIVED, &cb_rcvsubscribe);
-  osip_set_message_callback(osip ,OSIP_NIST_NOTIFY_RECEIVED,    &cb_rcvnotify);
-  osip_set_message_callback(osip ,OSIP_NIST_UNKNOWN_REQUEST_RECEIVED, &cb_sndunkrequest);
- *
- * </code></pre>
- * <P>
- * <H2>Step 2: Initialising a new transaction.</H2>
- * <BR>Let's assume you want to implement a User Agent and you want to
- * start a REGISTER transaction. Using the parser library, you will first
- * have to build a SIP compliant message. (oSIP, as a low layer library
- * provides an interface to build SIP messages, but it's up to you to
- * correctly fill all the required fields.)
- * As soon as you have build the SIP message, you are ready to start a new
- * transaction. Here is the code:
- * <BR><pre><code>
- * osip_t *osip       = your_global_osip_context;
- * osip_transaction_t *transaction;
- * osip_message_t     *sip_register_message;
- * osip_event_t       *sipevent;
- *
- * application_build_register(&sip_register_message);
- * osip_transaction_init(&transaction,
- *		   NICT, //a REGISTER is a Non-Invite-Client-Transaction
- *		   osip,
- *		   sip_register_message);
- *
- * // If you have a special context that you want to associate to that
- * // transaction, you can use a special method that associate your context
- * // to the transaction context.
- *
- * osip_transaction_set_your_instance(transaction, any_pointer);
- *
- * // at this point, the transaction context exists in oSIP but you still have
- * // to give the SIP message to the finite state machine.   
- * sipevent = osip_new_outgoing_sipmessage(msg);
- * sipevent->transactionid =  transaction->transactionid;
- * osip_transaction_add_event(transaction, sipevent);
- * // at this point, the event will be handled by oSIP. (The memory resource will
- * // also be handled by oSIP). Note that no action is taken there. 
- * </pre></code>
- * <P>Adding new events in the fsm is made with similar code.
- * <P>
- * <H2>Step 3: Consuming events.</H2>
- * <BR>The previous step show how to create a transaction and one possible way
- * to add a new event. (Note, that some events -the TIMEOUT_* ones- will be
- * added by oSIP not by the application). In this step, we describe how the
- * oSIP stack will consume events. In fact, this is very simple, but you
- * should be aware that it's not always allowed to consume an event at any time!
- * The fsm MUST consume events sequentially within a transaction. This means
- * that when your are calling osip_transaction_execute(), it is forbidden to call
- * this method again with the same transaction context until the first call
- * has returned. In a multi threaded application, if one thread handles one
- * transaction, the code will be the following:
- * <BR><pre><code>
- *   while (1)
- *     {
- *       se = (osip_event_t *)osip_fifo_get(transaction->transactionff);
- *       if (se==NULL)
- * 	  osip_thread_exit();
- *       if (osip_transaction_execute(transaction,se)<1)  // deletion asked
- * 	  osip_thread_exit();
- *   }
- * </pre></code>
- * <P>
- * <H2>Step 4: How the stack will announce the events</H2>
- * Looking at the case of a usual outgoing REGISTER transaction, this behaviour
- * is expected.
- * <BR>When an event is seen as useful for the fsm, it means that a transition
- * from one state to another has to be done on the transaction context. If the
- * event is SND_REQUEST (this is the case for an outgoing REGISTER), the
- * callback previously registered to announce this action will be called. This
- * callback is useless for the application as no action has to be taken at this
- * step. A more interesting announcement will be made when consuming the
- * first final response received. If the callbacks associated to 2xx message
- * is called, then the transaction has succeeded. Inside this callback, you
- * will probably inform the user of the success of the registration if you want
- * to do so...
- * If the final response is not a 2xx, or the network callback is called, you'll
- * probably want to take some actions. For example, if you receive a 302, you'll
- * probably want to retry a registration at the new location. All that decision
- * is up to you.
  */
 
 /**
  * @defgroup oSIP_FSM oSIP fsm Handling
- * @ingroup oSIP
+ * @ingroup osip2_fsm
  * @{
  */
 
@@ -434,113 +194,127 @@ extern "C"
 
 /**
  * Structure for INVITE CLIENT TRANSACTION (outgoing INVITE transaction).
- * @defvar osip_ict_t
+ * @var osip_ict_t
  */
   typedef struct osip_ict osip_ict_t;
 
+/**
+ * Structure for INVITE CLIENT TRANSACTION.
+ * @struct osip_ict
+ */
   struct osip_ict
   {
-    /* state machine is implied... */
-    int timer_a_length;		/* A=T1, A=2xT1... (unreliable tr only)  */
-    struct timeval timer_a_start;
-    int timer_b_length;		/* B = 64* T1                            */
-    struct timeval timer_b_start;	/* fire when transaction timeouts        */
-    int timer_d_length;		/* D >= 32s for unreliable tr (or 0)     */
-    struct timeval timer_d_start;	/* should be equal to timer H            */
-    char *destination;		/* url used to send requests         */
-    int port;			/* port of next hop                  */
-
+    int timer_a_length;		  /**@internal A=T1, A=2xT1... (unreliable only) */
+    struct timeval timer_a_start; /**@internal */
+    int timer_b_length;		  /**@internal B = 64* T1 */
+    struct timeval timer_b_start; /**@internal fire when transaction timeouts */
+    int timer_d_length;		  /**@internal D >= 32s for unreliable tr (or 0) */
+    struct timeval timer_d_start; /**@internal should be equal to timer H */
+    char *destination;	          /**@internal url used to send requests */
+    int port;                     /**@internal port of next hop */
   };
 
 /**
  * Structure for NON-INVITE CLIENT TRANSACTION (outgoing NON-INVITE transaction).
- * @defvar osip_nict_t
+ * @var osip_nict_t
  */
   typedef struct osip_nict osip_nict_t;
 
+/**
+ * Structure for NON-INVITE CLIENT TRANSACTION.
+ * @struct osip_nict
+ */
   struct osip_nict
   {
-    /* state machine is implied... */
-
-    int timer_e_length;		/* A=T1, A=2xT1... (unreliable tr. only) */
-    struct timeval timer_e_start;	/*  (else = -1) not active               */
-    int timer_f_length;		/* B = 64* T1                            */
-    struct timeval timer_f_start;	/* fire when transaction timeouts        */
-    int timer_k_length;		/* K = T4 (else = 0)                     */
-    struct timeval timer_k_start;
-    char *destination;		/* url used to send requests         */
-    int port;			/* port of next hop                  */
+    int timer_e_length;		  /**@internal A=T1, A=2xT1... (unreliable only) */
+    struct timeval timer_e_start; /**@internal */
+    int timer_f_length;		  /**@internal B = 64* T1 */
+    struct timeval timer_f_start; /**@internal fire when transaction timeouts */
+    int timer_k_length;		  /**@internal K = T4 (else = 0) */
+    struct timeval timer_k_start; /**@internal */
+    char *destination;		  /**@internal url used to send requests */
+    int port;			  /**@internal port of next hop */
 
   };
 
 /**
  * Structure for INVITE SERVER TRANSACTION (incoming INVITE transaction).
- * @defvar osip_ist_t
+ * @var osip_ist_t
  */
   typedef struct osip_ist osip_ist_t;
 
+/**
+ * Structure for INVITE SERVER TRANSACTION.
+ * @struct osip_ist
+ */
   struct osip_ist
   {
-    int timer_g_length;		/* G=MIN(T1*2,T2) for unreliable trans.  */
-    struct timeval timer_g_start;	/* 0 when reliable transport is used!    */
-    int timer_h_length;		/* H = 64* T1                            */
-    struct timeval timer_h_start;	/* fire when if no ACK is received       */
-    int timer_i_length;		/* I = T4 for unreliable transport (or 0) */
-    struct timeval timer_i_start;	/* absorb all ACK                        */
+    int timer_g_length;	     /**@internal G=MIN(T1*2,T2) for unreliable trans. */
+    struct timeval timer_g_start; /**@internal 0 when reliable transport is used */
+    int timer_h_length;		  /**@internal H = 64* T1 */
+    struct timeval timer_h_start; /**@internal fire when if no ACK is received */
+    int timer_i_length;		  /**@internal I = T4 for unreliable (or 0) */
+    struct timeval timer_i_start; /**@internal absorb all ACK */
   };
 
 /**
  * Structure for NON-INVITE SERVER TRANSACTION (incoming SERVER transaction).
- * @defvar osip_nist_t
+ * @var osip_nist_t
  */
   typedef struct osip_nist osip_nist_t;
 
+/**
+ * Structure for NON-INVITE SERVER TRANSACTION.
+ * @struct osip_nist
+ */
   struct osip_nist
   {
-    int timer_j_length;		/* J = 64*T1 (else 0) */
-    struct timeval timer_j_start;
+    int timer_j_length;		   /**@internal J = 64*T1 (else 0) */
+    struct timeval timer_j_start;  /**@internal */
   };
 
 /**
  * Structure for transaction handling.
- * @defvar osip_transaction_t
+ * @var osip_transaction_t
  */
   typedef struct osip_transaction osip_transaction_t;
 
+/**
+ * Structure for transaction handling
+ * @struct osip_transaction
+ */
   struct osip_transaction
   {
 
-    void *your_instance;	/* add whatever you want here.       */
-    int transactionid;		/* simple id used to identify the tr. */
-    osip_fifo_t *transactionff;	/* events must be added in this fifo */
+    void *your_instance;	/**< User Defined Pointer. */
+    int transactionid;		/**< Internal Transaction Identifier. */
+    osip_fifo_t *transactionff;	/**< events must be added in this fifo */
 
-    osip_via_t *topvia;		/* CALL-LEG definition */
-    osip_from_t *from;		/* CALL-LEG definition */
-    osip_to_t *to;
-    osip_call_id_t *callid;
-    osip_cseq_t *cseq;
+    osip_via_t *topvia;		/**< CALL-LEG definition (Top Via) */
+    osip_from_t *from;		/**< CALL-LEG definition (From)    */
+    osip_to_t *to;              /**< CALL-LEG definition (To)      */
+    osip_call_id_t *callid;     /**< CALL-LEG definition (Call-ID) */
+    osip_cseq_t *cseq;          /**< CALL-LEG definition (CSeq)    */
 
-    osip_message_t *orig_request;	/* last request sent                 */
-    osip_message_t *last_response;	/* last response received            */
-    osip_message_t *ack;	/* ack request sent                  */
+    osip_message_t *orig_request;  /**< Initial request            */
+    osip_message_t *last_response; /**< Last response              */
+    osip_message_t *ack;	   /**< ack request sent           */
 
-    state_t state;		/* state of transaction              */
+    state_t state;		/**< Current state of the transaction */
 
-    time_t birth_time;		/* birth_date of transaction         */
-    time_t completed_time;	/* end   date of transaction         */
+    time_t birth_time;		/**< birth date of transaction        */
+    time_t completed_time;	/**< end   date of transaction        */
 
-    /* RESPONSE are received on this socket */
-    int in_socket;
-    /* REQUESTS are sent on this socket */
-    int out_socket;
+    int in_socket;              /**< Optional socket for incoming message */
+    int out_socket;             /**< Optional place for outgoing message */
 
-    void *config;		/* transaction is managed by config  */
+    void *config;		/**@internal transaction is managed by osip_t  */
 
-    osip_fsm_type_t ctx_type;
-    osip_ict_t *ict_context;
-    osip_ist_t *ist_context;
-    osip_nict_t *nict_context;
-    osip_nist_t *nist_context;
+    osip_fsm_type_t ctx_type;   /**< Type of the transaction */
+    osip_ict_t *ict_context;    /**@internal */
+    osip_ist_t *ist_context;    /**@internal */
+    osip_nict_t *nict_context;  /**@internal */
+    osip_nist_t *nist_context;  /**@internal */
   };
 
 
@@ -612,6 +386,9 @@ extern "C"
     OSIP_NIST_STATUS_6XX_SENT,              /**< 6XX FOR MESSAGE SENT */
     OSIP_NIST_STATUS_3456XX_SENT_AGAIN,     /**< RESPONSE RETRANSMITTED */
 
+    OSIP_ICT_STATUS_TIMEOUT,                /**< TIMER B EXPIRATION: NO REMOTE ANSWER  */
+    OSIP_NICT_STATUS_TIMEOUT,               /**< TIMER F EXPIRATION: NO REMOTE ANSWER  */
+
     OSIP_MESSAGE_CALLBACK_COUNT             /**< END OF ENUM */
   } osip_message_callback_type_t;
 
@@ -630,6 +407,7 @@ extern "C"
 
 /**
  * Enumeration for callback type used when a transport error is detected.
+ * @enum osip_transport_error_callback_type_t
  */
   typedef enum osip_transport_error_callback_type
   {
@@ -641,64 +419,84 @@ extern "C"
     OSIP_TRANSPORT_ERROR_CALLBACK_COUNT   /**< END OF ENUM */
   } osip_transport_error_callback_type_t;
 
+/**
+ * Callback definition for message announcements.
+ * @var osip_message_cb_t
+ */
   typedef void (*osip_message_cb_t) (int type, osip_transaction_t *,
 				     osip_message_t *);
+/**
+ * Callback definition for end of transaction announcements.
+ * @var osip_kill_transaction_cb_t
+ */
   typedef void (*osip_kill_transaction_cb_t) (int type, osip_transaction_t *);
+/**
+ * Callback definition for transport error announcements.
+ * @var osip_transport_error_cb_t
+ */
   typedef void (*osip_transport_error_cb_t) (int type, osip_transaction_t *,
 					     int error);
 
 
-#ifdef OSIP_RETRANSMIT_2XX
   struct osip_dialog;
 
+/**
+ * Structure for 2XX retransmission management.
+ * @var ixt_t
+ */
   typedef struct ixt_t ixt_t;
 
+/**
+ * Structure for 2XX retransmission management.
+ * @struct ixt
+ */
   struct ixt_t
   {
     /* any ACK received that match this context will set counter to -1 */
-    struct osip_dialog *dialog;
-    osip_message_t *msg2xx;	/* copy of string to retransmit */
-    osip_message_t *ack;	/* useless for ist */
-    time_t start;
-    int interval;		/* between each retransmission, in ms */
-    char *dest;
-    int port;
-    int sock;
-    int counter;		/* start at 7 */
+    struct osip_dialog *dialog; /**< related dialog */
+    osip_message_t *msg2xx;     /**< buffer to retransmit */
+    osip_message_t *ack;        /**< ack message if needed */
+    time_t start;               /**< Time of first retransmission */
+    int interval;               /**< delay between retransmission, in ms */
+    char *dest;                 /**< destination host */
+    int port;                   /**< destination port */
+    int sock;                   /**< socket to use */
+    int counter;		/**< start at 7 */
   };
 
-#endif
 
 /**
  * Structure for osip handling.
  * In order to use osip, you have to manage at least one global instance
  * of an osip_t element. Then, you'll register a set of required callbacks
  * and a set of optional ones.
- * @defvar osip_t
+ * @var osip_t
  */
   typedef struct osip osip_t;
 
+/**
+ * Structure for osip handling.
+ * @struct osip
+ */
   struct osip
   {
 
-    void *application_context;	/* a pointer for your personnal usage */
+    void *application_context;	/**< User defined Pointer */
 
     /* list of transactions for ict, ist, nict, nist */
-    osip_list_t *osip_ict_transactions;
-    osip_list_t *osip_ist_transactions;
-    osip_list_t *osip_nict_transactions;
-    osip_list_t *osip_nist_transactions;
+    osip_list_t *osip_ict_transactions;  /**< list of ict transactions */
+    osip_list_t *osip_ist_transactions;  /**< list of ist transactions */
+    osip_list_t *osip_nict_transactions; /**< list of nict transactions */
+    osip_list_t *osip_nist_transactions; /**< list of nist transactions */
 
-    osip_list_t *ixt_retransmissions;	/* for retransmission of 2xx & ACK for INVITE */
+    osip_list_t *ixt_retransmissions;	 /**< list of ixt elements */
 
-    osip_message_cb_t msg_callbacks[OSIP_MESSAGE_CALLBACK_COUNT];
-    osip_kill_transaction_cb_t kill_callbacks[OSIP_KILL_CALLBACK_COUNT];
-    osip_transport_error_cb_t
-      tp_error_callbacks[OSIP_TRANSPORT_ERROR_CALLBACK_COUNT];
+    osip_message_cb_t msg_callbacks[OSIP_MESSAGE_CALLBACK_COUNT];        /**@internal */
+    osip_kill_transaction_cb_t kill_callbacks[OSIP_KILL_CALLBACK_COUNT]; /**@internal */
+    osip_transport_error_cb_t tp_error_callbacks[OSIP_TRANSPORT_ERROR_CALLBACK_COUNT];  /**@internal */
 
-    /* callbacks for sending messages */
     int (*cb_send_message) (osip_transaction_t *, osip_message_t *, char *,
-			    int, int);
+			    int, int);  /**@internal */
   };
 
 /**
@@ -728,20 +526,24 @@ extern "C"
 					 osip_transport_error_cb_t cb);
 
 /**
- * Structure for sipevent handling.
+ * Structure for osip event handling.
  * A osip_event_t element will have a type and will be related
  * to a transaction. In the general case, it is used by the
  * application layer to give SIP messages to the oSIP finite
  * state machine.
- * @defvar osip_event_t
+ * @var osip_event_t
  */
   typedef struct osip_event osip_event_t;
 
+/**
+ * Structure for osip event handling.
+ * @struct osip_event
+ */
   struct osip_event
   {
-    type_t type;
-    int transactionid;
-    osip_message_t *sip;
+    type_t type;             /**< Event Type */
+    int transactionid;       /**< identifier of the related osip transaction */
+    osip_message_t *sip;     /**< SIP message (optional) */
   };
 
 
@@ -770,6 +572,16 @@ extern "C"
  */
   int osip_transaction_free2 (osip_transaction_t * transaction);
 
+/**
+ * Search in a SIP response the destination where the message
+ * should be sent.
+ * @param response the message to work on.
+ * @param address a pointer to receive the allocated host address.
+ * @param portnum a pointer to receive the host port.
+ */
+  void osip_response_get_destination (osip_message_t * response,
+				      char **address,
+				      int *portnum);
 /**
  * Set the host and port destination used for sending the SIP message.
  * This can be useful for an application with 'DIRECT ROOTING MODE'
@@ -840,7 +652,6 @@ extern "C"
   int osip_transaction_get_destination (osip_transaction_t * transaction,
 					char **ip, int *port);
 
-#ifndef DOXYGEN
 
 /**
  * Set the socket for incoming message.
@@ -860,24 +671,6 @@ extern "C"
 				       int sock);
 
 
-#if 0
-
-
-/**
- * Check if the first 2 parameters match the other ones.
- * NOTE: THIS IS AN INTERNAL METHOD ONLY
- * @param to1 The initial to header.
- * @param from1 The initial from header.
- * @param to2 The new to header.
- * @param from2 The new from header.
- */
-  int callleg_match (osip_to_t * to1, osip_from_t * from1, osip_to_t * to2,
-		     osip_from_t * from2);
-
-#endif
-
-#endif				/* endif DOXYGEN */
-
 
 /** 
  * Allocate an osip_t element.
@@ -1013,12 +806,15 @@ extern "C"
 /**
  * Create a sipevent from a SIP message string.
  * @param buf The SIP message as a string.
+ * @param length The length of the buffer to parse.
  */
-  osip_event_t *osip_parse (char *buf);
+  osip_event_t *osip_parse (const char *buf, size_t length);
 
 
-#ifdef OSIP_RETRANSMIT_2XX
-
+/**
+ * Send required retransmissions
+ * @param osip The element to work on.
+ */
   void osip_retransmissions_execute (osip_t * osip);
 
 /**
@@ -1038,7 +834,7 @@ extern "C"
  * @param dialog The dialog the ACK is part of.
  * @param ack The ACK that has just been sent in response to a 200 Ok.
  * @param dest The destination host.
- * @param sock The destination port.
+ * @param port The destination port.
  * @param sock The socket to be used to send the message. (optional).
  */
   void osip_start_ack_retransmissions (osip_t * osip,
@@ -1051,7 +847,7 @@ extern "C"
  * @param osip The osip_t structure.
  * @param ack  The ack that has just been received.
  */
-  void osip_stop_200ok_retransmissions (osip_t * osip, osip_message_t * ack);
+  struct osip_dialog *osip_stop_200ok_retransmissions (osip_t * osip, osip_message_t * ack);
 
 /**
  * Stop out of fsm retransmissions (ACK or 200 Ok) associated to a given dialog.
@@ -1063,7 +859,6 @@ extern "C"
   void osip_stop_retransmissions_from_dialog (osip_t * osip,
 					      struct osip_dialog *dialog);
 
-#endif
 
 /**
  * Allocate a sipevent (we know this message is an OUTGOING SIP message).
