@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -35,13 +35,17 @@
 
 /**
  * Structure for Content-Length headers.
- * @defvar osip_content_length_t
+ * @var osip_content_length_t
  */
   typedef struct osip_content_length osip_content_length_t;
 
+/**
+ * Definition of the Content-Length header.
+ * @struct osip_content_length
+ */
   struct osip_content_length
   {
-    char *value;
+    char *value;    /**< value for Content-Length (size of attachments) */
   };
 
 #ifdef __cplusplus
