@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -36,18 +36,22 @@
 
 /**
  * Structure for Via headers.
- * @defvar osip_via_t
+ * @var osip_via_t
  */
   typedef struct osip_via osip_via_t;
 
+/**
+ * Definition of the Via header.
+ * @struct osip_via
+ */
   struct osip_via
   {
-    char *version;
-    char *protocol;
-    char *host;
-    char *port;
-    char *comment;
-    osip_list_t *via_params;
+    char *version;              /**< SIP Version */
+    char *protocol;             /**< Protocol used by SIP Agent */
+    char *host;                 /**< Host where to send answers */
+    char *port;                 /**< Port where to send answers */
+    char *comment;              /**< Comments about SIP Agent */
+    osip_list_t *via_params;    /**< Via parameters */
   };
 
 #ifdef __cplusplus
@@ -88,55 +92,65 @@ extern "C"
  * @param header The element to work on.
  * @param value The value of the element.
  */
+#define osip_via_set_version via_set_version
   void via_set_version (osip_via_t * header, char *value);
 /**
  * Get the SIP version from a Via header.
  * @param header The element to work on.
  */
+#define osip_via_get_version via_get_version
   char *via_get_version (osip_via_t * header);
 /**
  * Set the protocol in the Via element.
  * @param header The element to work on.
  * @param value The value of the element.
  */
+#define osip_via_set_protocol via_set_protocol
   void via_set_protocol (osip_via_t * header, char *value);
 /**
  * Get the protocol from a Via header.
  * @param header The element to work on.
  */
+#define osip_via_get_protocol via_get_protocol
   char *via_get_protocol (osip_via_t * header);
 /**
  * Set the host in the Via element.
  * @param header The element to work on.
  * @param value The value of the element.
  */
+#define osip_via_set_host via_set_host
   void via_set_host (osip_via_t * header, char *value);
 /**
  * Get the host from a Via header.
  * @param header The element to work on.
  */
+#define osip_via_get_host via_get_host
   char *via_get_host (osip_via_t * header);
 /**
  * Set the port in the Via element.
  * @param header The element to work on.
  * @param value The value of the element.
  */
+#define osip_via_set_port via_set_port
   void via_set_port (osip_via_t * header, char *value);
 /**
  * Get the port from a Via header.
  * @param header The element to work on.
  */
+#define osip_via_get_port via_get_port
   char *via_get_port (osip_via_t * header);
 /**
  * Set the comment in the Via element.
  * @param header The element to work on.
  * @param value The value of the element.
  */
+#define osip_via_set_comment via_set_comment
   void via_set_comment (osip_via_t * header, char *value);
 /**
  * Get the comment from a Via header.
  * @param header The element to work on.
  */
+#define osip_via_get_comment via_get_comment
   char *via_get_comment (osip_via_t * header);
 
 /**
