@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -38,7 +38,6 @@
 
 /**
  * @defgroup oSIP_LIST oSIP list Handling
- * @ingroup oSIP
  * @{
  */
 
@@ -50,34 +49,57 @@ extern "C"
 #ifndef DOXYGEN
 /**
  * Structure for referencing a node in a osip_list_t element.
- * @defvar __node_t
+ * @var __node_t
  */
   typedef struct __node __node_t;
 
+/**
+ * Structure for referencing a node in a osip_list_t element.
+ * @struct __node
+ */
   struct __node
   {
-    void *next;			/* next __node_t */
-    void *element;
+    void *next;			/**< next __node_t containing element */
+    void *element;              /**< element in Current node */
   };
 #endif
 
 /**
  * Structure for referencing a list of elements.
- * @defvar osip_list_t
+ * @var osip_list_t
  */
   typedef struct osip_list osip_list_t;
 
+/* added by bennewit@cs.tu-berlin.de */
+  typedef struct {
+    __node_t * actual;
+    __node_t ** prev;
+    osip_list_t * li;
+    int pos;
+  } osip_list_iterator_t;
+
+/**
+ * Structure for referencing a list of elements.
+ * @struct osip_list
+ */
   struct osip_list
   {
 
-    int nb_elt;
-    __node_t *node;
+    int nb_elt;         /**< Number of element in the list */
+    __node_t *node;     /**< Next node containing element  */
 
   };
 
+/* added by bennewit@cs.tu-berlin.de */
+#define osip_list_iterator_has_elem( it ) ( 0 != (it).actual && (it).pos < (it).li->nb_elt )
+
 /**
  * Initialise a osip_list_t element.
- * NOTE: this element MUST be previously allocated.
+ * NOTE: this element MUST be previously allocated with
+ * osip_malloc(). The osip_free() call on the list is
+ * still automatically done by osip_list_free(). This
+ * also means you can't use a static osip_list_t variable
+ * if you want to use osip_list_free().
  * @param li The element to initialise.
  */
   int osip_list_init (osip_list_t * li);
@@ -124,6 +146,12 @@ extern "C"
  */
   int osip_list_remove (osip_list_t * li, int pos);
 
+/* added by bennewit@cs.tu-berlin.de */
+  void * osip_list_get_first( osip_list_t * li, osip_list_iterator_t * it );
+/* added by bennewit@cs.tu-berlin.de */
+  void * osip_list_get_next( osip_list_iterator_t * it );
+/* added by bennewit@cs.tu-berlin.de */
+  void * osip_list_iterator_remove( osip_list_iterator_t * it );
 
 #ifdef __cplusplus
 }
