@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -30,41 +30,11 @@
  * @brief oSIP SIP Message Accessor Routines
  *
  * This is the SIP accessor and parser related API.
- * <BR>Understanding the parser implementation will prevent you from
- * using it improperly. Read this carefully.<BR>
- * <BR>This implementation could be seen as a partial implementation
- * of the whole SIP syntax. In other words, the parser is 'tolerant'
- * and will not detect a lot of error cases. As an example, no error
- * will be detected while trying to parse the following request-uri:
- * <BR><pre><code>INVITE sip:jack@atosc.org:abcd SIP/2.0</pre></code>
- * <BR>This code shows that even if your SIP message is parsed
- * correctly by oSIP, it may still be not compliant. This could be
- * used by attackers to make your application crash or whatever.
- * In this example, if you are trying to call the atoi() method with
- * the string 'abcd', your application will crash. Of course, there
- * exist solutions! You can check yourself for the validity of the
- * string or use the strtol() method (found on most unix) which is
- * capable of detecting such error cases.
- * <BR>Are you wondering why the parser has been built this way?
- * <BR>The initial answer is that each SIP application have
- * different requirement and some (the proxy!) needs SIP message
- * to be parsed as quickly as possible. Also, most applications
- * only need a few information from a SIP message. (the first Via
- * is the only one interesting!). If the parser was fully checking each
- * Via field validity, it would consume too much CPU on useless
- * operations. If you think this model does not fit your application,
- * then you should buy a slow stack :-).
- * <BR>Is there any plan to change that behaviour?
- * <BR>I do not need it, but if this interest you, it would be
- * possible to compile oSIP in 2 different ways: a full checker
- * model could be useful for SIP application with no performance
- * requirements. Any contributions is welcomed and will be merged
- * if it's made optional.
  */
 
 /**
- * @defgroup oSIP_SMSG oSIP message API
- * @ingroup oSIP
+ * @defgroup oSIP_MESSAGE oSIP message API
+ * @ingroup osip2_parser
  * @{
  */
 
@@ -75,57 +45,63 @@ extern "C"
 
 /**
  * Structure for SIP Message (REQUEST and RESPONSE).
- * @defvar osip_message_t
+ * @var osip_message_t
  */
   typedef struct osip_message osip_message_t;
 
+/**
+ * Structure for SIP Message (REQUEST and RESPONSE).
+ * @struct osip_message
+ */
   struct osip_message
   {
-    /* Start-Line definition */
-    char *sip_version;
-    /* req */
-    osip_uri_t *req_uri;
-    char *sip_method;
-    /* resp */
-    int status_code;
-    char *reason_phrase;
-    /* End of Start-Line definition */
+    char *sip_version;                /**< SIP version (SIP request only) */
+    osip_uri_t *req_uri;              /**< Request-Uri (SIP request only) */
+    char *sip_method;                 /**< METHOD (SIP request only) */
 
-    osip_list_t *accepts;
-    osip_list_t *accept_encodings;
-    osip_list_t *accept_languages;
-    osip_list_t *alert_infos;
-    osip_list_t *allows;
-    osip_list_t *authorizations;
-    osip_call_id_t *call_id;
-    osip_list_t *call_infos;
-    osip_list_t *contacts;
-    osip_list_t *content_dispositions;
-    osip_list_t *content_encodings;
-    osip_content_length_t *content_length;
-    osip_content_type_t *content_type;
-    osip_cseq_t *cseq;
-    osip_list_t *error_infos;
-    osip_from_t *from;
-    osip_mime_version_t *mime_version;
-    osip_list_t *proxy_authenticates;
-    osip_list_t *proxy_authorizations;
-    osip_list_t *record_routes;
-    osip_list_t *routes;
-    osip_to_t *to;
-    osip_list_t *vias;
-    osip_list_t *www_authenticates;
+    int status_code;                  /**< Status Code (SIP answer only) */
+    char *reason_phrase;              /**< Reason Phrase (SIP answer only) */
 
-    osip_list_t *headers;
+    osip_list_t *accepts;             /**< Accept headers */
+    osip_list_t *accept_encodings;    /**< Accept-Encoding headers */
+    osip_list_t *accept_languages;    /**< Accept-Language headers */
+    osip_list_t *alert_infos;         /**< Alert-Info headers */
+    osip_list_t *allows;              /**< Allows headers */
+    osip_list_t *authentication_infos;/**< authentication_info headers */
+    osip_list_t *authorizations;      /**< Authorizations headers */
+    osip_call_id_t *call_id;          /**< Call-ID header */
+    osip_list_t *call_infos;          /**< Call-Infos header */
+    osip_list_t *contacts;            /**< Contacts headers */
+    osip_list_t *content_dispositions;/**< Content-Dispositions headers */
+    osip_list_t *content_encodings;   /**< Content-Encodings headers */
+    osip_content_length_t *content_length;   /**< Content-Length header */
+    osip_content_type_t *content_type;       /**< Content-Type header */
+    osip_cseq_t *cseq;                /**< CSeq header */
+    osip_list_t *error_infos;         /**< Error-Info headers */
+    osip_from_t *from;                /**< From header */
+    osip_mime_version_t *mime_version;/**< Mime-Version header */
+    osip_list_t *proxy_authenticates; /**< Proxy-Authenticate headers */
+    osip_list_t *proxy_authentication_infos; /**< P-Authentication-Info headers */
+    osip_list_t *proxy_authorizations;/**< Proxy-authorization headers */
+    osip_list_t *record_routes;       /**< Record-Route headers */
+    osip_list_t *routes;              /**< Route headers */
+    osip_to_t *to;                    /**< To header */
+    osip_list_t *vias;                /**< Vias headers */
+    osip_list_t *www_authenticates;   /**< WWW-Authenticate headers */
 
-    osip_list_t *bodies;
+    osip_list_t *headers;             /**< Other headers */
+
+    osip_list_t *bodies;              /**< List of attachements */
 
     /*
        1: structure and buffer "message" are identical.
        2: buffer "message" is not up to date with the structure info (call osip_message_to_str to update it).
      */
-    int message_property;
-    char *message;
+    int message_property;             /**@internal */
+    char *message;                    /**@internal */
+    size_t message_length;            /**@internal */
+
+    void *application_data;           /**can be used by upper layer*/
   };
 
 #ifndef SIP_MESSAGE_MAX_LENGTH
@@ -155,22 +131,17 @@ extern "C"
 /**
  * Parse a osip_message_t element.
  * @param sip The resulting element.
- * @param message The buffer to parse.
+ * @param buf The buffer to parse.
+ * @param length The length of the buffer to parse.
  */
-  int osip_message_parse (osip_message_t * sip, const char *message);
-/**
- * Parse a osip_message_t element.
- * @param sip The resulting element.
- * @param message The buffer to parse.
- * @param message The length of buffer to parse.
- */
-  int osip_message_parse2 (osip_message_t * sip, const char *buf, size_t length);
+  int osip_message_parse (osip_message_t * sip, const char *buf, size_t length);
 /**
  * Get a string representation of a osip_message_t element.
  * @param sip The element to work on.
  * @param dest new allocated buffer returned.
+ * @param message_length The length of the returned buffer.
  */
-  int osip_message_to_str (osip_message_t * sip, char **dest);
+  int osip_message_to_str (osip_message_t * sip, char **dest, size_t *message_length);
 /**
  * Clone a osip_message_t element.
  * @param sip The element to clone.
@@ -254,77 +225,89 @@ extern "C"
  * @param msg the SIP message.
  */
 #define MSG_IS_INVITE(msg)   (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"INVITE",6))
+			      0==strcmp((msg)->sip_method,"INVITE"))
 /**
  * Test if the message is an ACK REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_ACK(msg)      (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"ACK",3))
+			      0==strcmp((msg)->sip_method,"ACK"))
 /**
  * Test if the message is a REGISTER REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_REGISTER(msg) (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"REGISTER",8))
+			      0==strcmp((msg)->sip_method,"REGISTER"))
 /**
  * Test if the message is a BYE REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_BYE(msg)      (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"BYE",3))
+			      0==strcmp((msg)->sip_method,"BYE"))
 /**
  * Test if the message is an OPTIONS REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_OPTIONS(msg)  (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"OPTIONS",7))
+			      0==strcmp((msg)->sip_method,"OPTIONS"))
 /**
  * Test if the message is an INFO REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_INFO(msg)     (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"INFO",4))
+			      0==strcmp((msg)->sip_method,"INFO"))
 /**
  * Test if the message is a CANCEL REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_CANCEL(msg)   (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"CANCEL",6))
+			      0==strcmp((msg)->sip_method,"CANCEL"))
 /**
  * Test if the message is a REFER REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_REFER(msg)   (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"REFER",5))
+			      0==strcmp((msg)->sip_method,"REFER"))
 /**
  * Test if the message is a NOTIFY REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_NOTIFY(msg)   (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"NOTIFY",6))
+			      0==strcmp((msg)->sip_method,"NOTIFY"))
 /**
  * Test if the message is a SUBSCRIBE REQUEST
+ * @def MSG_IS_SUBSCRIBE
  * @param msg the SIP message.
  */
 #define MSG_IS_SUBSCRIBE(msg)  (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"SUBSCRIBE",9))
+			      0==strcmp((msg)->sip_method,"SUBSCRIBE"))
 /**
  * Test if the message is a MESSAGE REQUEST
  * @param msg the SIP message.
  */
 #define MSG_IS_MESSAGE(msg)  (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"MESSAGE",7))
+			      0==strcmp((msg)->sip_method,"MESSAGE"))
 /**
  * Test if the message is a PRACK REQUEST  (!! PRACK IS NOT SUPPORTED by the fsm!!)
  * @param msg the SIP message.
  */
 #define MSG_IS_PRACK(msg)    (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"PRACK",5))
+			      0==strcmp((msg)->sip_method,"PRACK"))
 
 
+/**
+ * Test if the message is an UPDATE REQUEST
+ * @param msg the SIP message.
+ */
 #define MSG_IS_UPDATE(msg)    (MSG_IS_REQUEST(msg) && \
-			      0==strncmp((msg)->sip_method,"UPDATE",6))
+			      0==strcmp((msg)->sip_method,"UPDATE"))
+
+/**
+ * Test if the message is an UPDATE REQUEST
+ * @param msg the SIP message.
+ */
+#define MSG_IS_PUBLISH(msg)    (MSG_IS_REQUEST(msg) && \
+			      0==strcmp((msg)->sip_method,"PUBLISH"))
 
 
 /**
