@@ -24,7 +24,16 @@
 /* Include necessary headers for osip */
 
 #include <stdio.h>
-#ifdef WIN32
+
+#if defined(__PALMOS__) && (__PALMOS__ >= 0x06000000)
+#	define STDC_HEADERS 1
+#	define HAVE_CTYPE_H 1
+#	define HAVE_STRING_H 1
+#	define HAVE_SYS_TYPES_H 1
+#	define HAVE_TIME_H 1
+#	define HAVE_STDARG_H 1
+
+#elif defined(WIN32)
 
 #define STDC_HEADERS 1
 #define HAVE_CTYPE_H 1
@@ -54,12 +63,22 @@
 #endif
 
 #ifdef __VXWORKS_OS__
-
 #include <string.h>
 #include <time.h>
+#include <sys/times.h>
 #include <stdarg.h>
+#include <sys/types.h>
+#include <stdlib.h>
 #define VA_START(a, f)  va_start(a, f)
 
+/* VxWorks lacks support for snprintf */
+int osip_vsnprintf( char* buf, int max, const char *fmt, va_list ap);
+int osip_snprintf(  char *buf, int max, const char *fmt, ...);
+
+#define snprintf  osip_snprintf
+#define vsnprintf osip_vsnprintf
+
+
 #else /* end of __VXWORKS_OS__ */
 
 #if defined (HAVE_CONFIG_H)
