@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -37,7 +37,7 @@
 
 /**
  * @defgroup oSIP_SDP oSIP SDP parser Handling
- * @ingroup oSIP
+ * @ingroup osip2_sdp
  * @{
  */
 
@@ -48,14 +48,18 @@ extern "C"
 
 /**
  * Structure for referencing bandwidth header.
- * @defvar sdp_bandwidth_t
+ * @var sdp_bandwidth_t
  */
   typedef struct sdp_bandwidth sdp_bandwidth_t;
 
+/**
+ * SDP bandwidth definition.
+ * @struct sdp_bandwidth
+ */
   struct sdp_bandwidth
   {
-    char *b_bwtype;
-    char *b_bandwidth;
+    char *b_bwtype;             /**< bandwidth type */
+    char *b_bandwidth;          /**< bandwidth value */
   };
 
 /**
@@ -71,15 +75,19 @@ extern "C"
 
 /**
  * Structure for referencing time description header.
- * @defvar sdp_time_descr_t
+ * @var sdp_time_descr_t
  */
   typedef struct sdp_time_descr sdp_time_descr_t;
 
+/**
+ * SDP Time description definition.
+ * @struct sdp_time_descr
+ */
   struct sdp_time_descr
   {
-    char *t_start_time;
-    char *t_stop_time;
-    osip_list_t *r_repeats;	/* list of char * */
+    char *t_start_time;            /**< start time */
+    char *t_stop_time;             /**< stop time */
+    osip_list_t *r_repeats;        /**< repeat headers */
   };
 
 /**
@@ -95,14 +103,18 @@ extern "C"
 
 /**
  * Structure for referencing key header.
- * @defvar sdp_key_t
+ * @var sdp_key_t
  */
   typedef struct sdp_key sdp_key_t;
 
+/**
+ * SDP key definition.
+ * @struct sdp_key
+ */
   struct sdp_key
   {
-    char *k_keytype;		/* "prompt", "clear", "base64", "uri" */
-    char *k_keydata;		/* key data for "clear" and "base64", uri for "uri" */
+    char *k_keytype;	/**< Key Type (prompt, clear, base64, uri) */
+    char *k_keydata;	/**< key data */
   };
 
 /**
@@ -118,14 +130,18 @@ extern "C"
 
 /**
  * Structure for referencing an attribute header.
- * @defvar sdp_attribute_t
+ * @var sdp_attribute_t
  */
   typedef struct sdp_attribute sdp_attribute_t;
 
+/**
+ * SDP attribute definition.
+ * @struct sdp_attribute
+ */
   struct sdp_attribute
   {
-    char *a_att_field;
-    char *a_att_value;		/* optional */
+    char *a_att_field;          /**< attribute field */
+    char *a_att_value;	        /**< attribute value (optional) */
   };
 
 /**
@@ -142,17 +158,21 @@ extern "C"
 
 /**
  * Structure for referencing a connection header.
- * @defvar sdp_connection_t
+ * @var sdp_connection_t
  */
   typedef struct sdp_connection sdp_connection_t;
 
+/**
+ * SDP connection definition.
+ * @struct sdp_connection
+ */
   struct sdp_connection
   {
-    char *c_nettype;
-    char *c_addrtype;
-    char *c_addr;
-    char *c_addr_multicast_ttl;
-    char *c_addr_multicast_int;
+    char *c_nettype;             /**< Network Type */
+    char *c_addrtype;            /**< Network Address Type */
+    char *c_addr;                /**< Address */
+    char *c_addr_multicast_ttl;	 /**< TTL value for multicast address */
+    char *c_addr_multicast_int;	 /**< Number of multicast address */
   };
 
 /**
@@ -168,23 +188,27 @@ extern "C"
 
 /**
  * Structure for referencing a media header.
- * @defvar sdp_media_t
+ * @var sdp_media_t
  */
   typedef struct sdp_media sdp_media_t;
 
+/**
+ * SDP media definition.
+ * @struct sdp_media
+ */
   struct sdp_media
   {
-    char *m_media;
-    char *m_port;
-    char *m_number_of_port;
-    char *m_proto;
-    osip_list_t *m_payloads;
+    char *m_media;              /**< media type */
+    char *m_port;               /**< port number */
+    char *m_number_of_port;     /**< number of port */
+    char *m_proto;              /**< protocol to be used */
+    osip_list_t *m_payloads;    /**< list of payloads (as strings) */
 
-    char *i_info;
-    osip_list_t *c_connections;	/* list of sdp_connection_t * */
-    osip_list_t *b_bandwidths;	/* list of sdp_bandwidth_t * */
-    osip_list_t *a_attributes;	/* list of sdp_attribute_t * */
-    sdp_key_t *k_key;
+    char *i_info;               /**< information header */
+    osip_list_t *c_connections;	/**< list of sdp_connection_t * */
+    osip_list_t *b_bandwidths;	/**< list of sdp_bandwidth_t * */
+    osip_list_t *a_attributes;	/**< list of sdp_attribute_t * */
+    sdp_key_t *k_key;           /**< key informations */
   };
 
 /**
@@ -200,31 +224,35 @@ extern "C"
 
 /**
  * Structure for referencing a SDP packet.
- * @defvar sdp_message_t
+ * @var sdp_message_t
  */
   typedef struct sdp_message sdp_message_t;
 
+/**
+ * SDP message definition.
+ * @struct sdp_message
+ */
   struct sdp_message
   {
-    char *v_version;
-    char *o_username;
-    char *o_sess_id;
-    char *o_sess_version;
-    char *o_nettype;
-    char *o_addrtype;
-    char *o_addr;
-    char *s_name;
-    char *i_info;
-    char *u_uri;
-    osip_list_t *e_emails;	/* list of char * */
-    osip_list_t *p_phones;	/* list of char * */
-    sdp_connection_t *c_connection;
-    osip_list_t *b_bandwidths;	/* list of sdp_bandwidth_t * */
-    osip_list_t *t_descrs;	/* list of sdp_time_descr_t * */
-    char *z_adjustments;
-    sdp_key_t *k_key;
-    osip_list_t *a_attributes;	/* list of sdp_attribute_t * */
-    osip_list_t *m_medias;	/* list of sdp_media_t * */
+    char *v_version;            /**< version header */
+    char *o_username;           /**< Username */
+    char *o_sess_id;            /**< Identifier for session */
+    char *o_sess_version;       /**< Version of session */
+    char *o_nettype;            /**< Network type */
+    char *o_addrtype;           /**< Address type */
+    char *o_addr;               /**< Address */
+    char *s_name;               /**< Subject header */
+    char *i_info;               /**< Information header */
+    char *u_uri;                /**< Uri header */
+    osip_list_t *e_emails;	/**< list of mail address */
+    osip_list_t *p_phones;	/**< list of phone numbers * */
+    sdp_connection_t *c_connection;   /**< Connection information */
+    osip_list_t *b_bandwidths;	/**< list of bandwidth info (sdp_bandwidth_t) */
+    osip_list_t *t_descrs;	/**< list of time description (sdp_time_descr_t) */
+    char *z_adjustments;        /**< Time adjustment header */
+    sdp_key_t *k_key;           /**< Key information header */
+    osip_list_t *a_attributes;	/**< list of global attributes (sdp_attribute_t) */
+    osip_list_t *m_medias;	/**< list of supported media (sdp_media_t) */
   };
 
 
@@ -354,12 +382,16 @@ extern "C"
  * @param value The token value.
  */
   int sdp_message_e_email_add (sdp_message_t * sdp, char *value);
+/**
+ * OBSOLETE: see sdp_message_e_email_get
+ * @def sdp_e_email_get
+ */
+  #define sdp_e_email_get sdp_message_e_email_get
 /**
  * Get one of the email ('e' field) of a SDP packet.
  * @param sdp The element to work on.
  * @param pos the index of the email line.
  */
-  #define sdp_e_email_get sdp_message_e_email_get
   char *sdp_message_e_email_get (sdp_message_t * sdp, int pos);
 /**
  * Set the version in a SDP packet.
@@ -549,6 +581,15 @@ extern "C"
  */
   int sdp_message_a_attribute_del (sdp_message_t * sdp, int pos_media, 
 				   char *att_field);
+/**
+ * delete one specific attribute fields specified by att_field.
+ * @param sdp The element to work on.
+ * @param pos_media The line number.
+ * @param att_field The value to remove.
+ * @param pos_attr The index of attribute to remove.
+ */
+  int sdp_message_a_attribute_del_at_index (sdp_message_t * sdp, int pos_media, 
+					    char *att_field, int pos_attr);
  /**
  * Get one of the attribute ('a' field) of a SDP packet.
  * @param sdp The element to work on.
@@ -602,6 +643,13 @@ extern "C"
  */
   char *sdp_message_m_port_get (sdp_message_t * sdp, int pos_media);
 /**
+ * Set the port number ('m' field) of a SDP packet.
+ * @param sdp The element to work on.
+ * @param pos_media The line number.
+ * @param port The new port to set (must be allocated with osip_malloc)
+ */
+  int sdp_message_m_port_set (sdp_message_t * sdp, int pos_media, char *port);
+ /**
  * Get the number of port ('m' field) of a SDP packet.
  * @param sdp The element to work on.
  * @param pos_media The line number.
@@ -629,6 +677,14 @@ extern "C"
  */
   char *sdp_message_m_payload_get (sdp_message_t * sdp, int pos_media,
 				   int pos);
+/**
+ * Remove a payload in a SDP packet.
+ * @param sdp The element to work on.
+ * @param pos_media The line number.
+ * @param pos The position of the payload in the media line.
+ */
+  int sdp_message_m_payload_del (sdp_message_t * sdp, int pos_media,
+ 				 int pos);
 
 
 /** @} */
