@@ -52,8 +52,9 @@ __osip_ict_init (osip_ict_t ** ict, osip_t * osip, osip_message_t * invite)
     if (proto == NULL)
       goto ii_error_1;
 
-    i = osip_strncasecmp (proto, "TCP", 3);
-    if (i != 0)
+    if (osip_strcasecmp (proto, "TCP") != 0
+	&& osip_strcasecmp (proto, "TLS") !=0
+	&& osip_strcasecmp (proto, "SCTP") !=0)
       {				/* for other reliable protocol than TCP, the timer
 				   must be desactived by the external application */
 	(*ict)->timer_a_length = DEFAULT_T1;
@@ -61,12 +62,12 @@ __osip_ict_init (osip_ict_t ** ict, osip_t * osip, osip_message_t * invite)
 	  (*ict)->timer_d_length = 32000;
 	else
 	  (*ict)->timer_d_length = 64 * DEFAULT_T1;
-	gettimeofday (&(*ict)->timer_a_start, NULL);
+	osip_gettimeofday (&(*ict)->timer_a_start, NULL);
 	add_gettimeofday (&(*ict)->timer_a_start, (*ict)->timer_a_length);
 	(*ict)->timer_d_start.tv_sec = -1;	/* not started */
       }
     else
-      {				/* TCP is used: */
+      {				/* reliable protocol is used: */
 	(*ict)->timer_a_length = -1;	/* A is not ACTIVE */
 	(*ict)->timer_d_length = 0;	/* MUST do the transition immediatly */
 	(*ict)->timer_a_start.tv_sec = -1;	/* not started */
@@ -77,6 +78,17 @@ __osip_ict_init (osip_ict_t ** ict, osip_t * osip, osip_message_t * invite)
   /* for PROXY, the destination MUST be set by the application layer,
      this one may not be correct. */
   osip_message_get_route (invite, 0, &route);
+  if (route != NULL && route->url!=NULL)
+    {
+      osip_uri_param_t *lr_param;
+      osip_uri_uparam_get_byname(route->url, "lr", &lr_param);
+      if (lr_param==NULL)
+	{
+	  /* using uncompliant proxy: destination is the request-uri */
+	  route = NULL;
+	}
+    }
+
   if (route != NULL)
     {
       int port = 5060;
@@ -86,10 +98,18 @@ __osip_ict_init (osip_ict_t ** ict, osip_t * osip, osip_message_t * invite)
       osip_ict_set_destination ((*ict), osip_strdup (route->url->host), port);
     }
   else
-    (*ict)->port = 5060;
+    {
+      int port = 5060;
+      
+      if (invite->req_uri->port != NULL)
+	port = osip_atoi (invite->req_uri->port);
+      osip_ict_set_destination ((*ict),
+				osip_strdup (invite->req_uri->host),
+				port);
+    }
 
   (*ict)->timer_b_length = 64 * DEFAULT_T1;
-  gettimeofday (&(*ict)->timer_b_start, NULL);
+  osip_gettimeofday (&(*ict)->timer_b_start, NULL);
   add_gettimeofday (&(*ict)->timer_b_start, (*ict)->timer_b_length);
 
   /* Oups! A bug! */
@@ -131,7 +151,7 @@ osip_event_t *
 __osip_ict_need_timer_a_event (osip_ict_t * ict, state_t state, int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ict == NULL)
     return NULL;
@@ -140,7 +160,7 @@ __osip_ict_need_timer_a_event (osip_ict_t * ict, state_t state, int transactioni
       /* may need timer A */
       if (ict->timer_a_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ict->timer_a_start, >))
+      if (osip_timercmp (&now, &ict->timer_a_start, >))
 	return __osip_event_new (TIMEOUT_A, transactionid);
     }
   return NULL;
@@ -151,7 +171,7 @@ __osip_ict_need_timer_b_event (osip_ict_t * ict, state_t state,
 			       int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ict == NULL)
     return NULL;
@@ -160,7 +180,7 @@ __osip_ict_need_timer_b_event (osip_ict_t * ict, state_t state,
       /* may need timer B */
       if (ict->timer_b_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ict->timer_b_start, >))
+      if (osip_timercmp (&now, &ict->timer_b_start, >))
 	return __osip_event_new (TIMEOUT_B, transactionid);
     }
   return NULL;
@@ -171,7 +191,7 @@ __osip_ict_need_timer_d_event (osip_ict_t * ict, state_t state,
 			       int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ict == NULL)
     return NULL;
@@ -180,7 +200,7 @@ __osip_ict_need_timer_d_event (osip_ict_t * ict, state_t state,
       /* may need timer D */
       if (ict->timer_d_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ict->timer_d_start, >))
+      if (osip_timercmp (&now, &ict->timer_d_start, >))
 	return __osip_event_new (TIMEOUT_D, transactionid);
     }
   return NULL;
