@@ -178,7 +178,7 @@ osip_ict_timeout_a_event (osip_transaction_t * ict, osip_event_t * evt)
 
   /* reset timer */
   ict->ict_context->timer_a_length = ict->ict_context->timer_a_length * 2;
-  gettimeofday (&ict->ict_context->timer_a_start, NULL);
+  osip_gettimeofday (&ict->ict_context->timer_a_start, NULL);
   add_gettimeofday (&ict->ict_context->timer_a_start,
 		    ict->ict_context->timer_a_length);
 
@@ -203,6 +203,7 @@ osip_ict_timeout_b_event (osip_transaction_t * ict, osip_event_t * evt)
   ict->ict_context->timer_b_length = -1;
   ict->ict_context->timer_b_start.tv_sec = -1;
 
+  __osip_message_callback (OSIP_ICT_STATUS_TIMEOUT, ict, evt->sip);
   __osip_transaction_set_state (ict, ICT_TERMINATED);
   __osip_kill_transaction_callback (OSIP_ICT_KILL_TRANSACTION, ict);
 }
@@ -346,6 +347,17 @@ ict_rcv_3456xx (osip_transaction_t * ict, osip_event_t * evt)
       if (ict->ict_context->destination == NULL)
 	{
 	  osip_message_get_route (ack, 0, &route);
+	  if (route != NULL && route->url!=NULL)
+	    {
+	      osip_uri_param_t *lr_param;
+	      osip_uri_uparam_get_byname(route->url, "lr", &lr_param);
+	      if (lr_param==NULL)
+		{
+		  /* using uncompliant proxy: destination is the request-uri */
+		  route = NULL;
+		}
+	    }
+
 	  if (route != NULL)
 	    {
 	      int port = 5060;
@@ -412,7 +424,7 @@ ict_rcv_3456xx (osip_transaction_t * ict, osip_event_t * evt)
    */
 
   /* start timer D (length is set to MAX (64*DEFAULT_T1 or 32000) */
-  gettimeofday (&ict->ict_context->timer_d_start, NULL);
+  osip_gettimeofday (&ict->ict_context->timer_d_start, NULL);
   add_gettimeofday (&ict->ict_context->timer_d_start,
 		    ict->ict_context->timer_d_length);
   __osip_transaction_set_state (ict, ICT_COMPLETED);
