@@ -47,8 +47,9 @@ __osip_ist_init (osip_ist_t ** ist, osip_t * osip, osip_message_t * invite)
     if (proto == NULL)
       goto ii_error_1;
 
-    i = osip_strncasecmp (proto, "TCP", 3);
-    if (i != 0)
+    if (osip_strcasecmp (proto, "TCP") != 0
+	&& osip_strcasecmp (proto, "TLS") !=0
+	&& osip_strcasecmp (proto, "SCTP") !=0)
       {				/* for other reliable protocol than TCP, the timer
 				   must be desactived by the external application */
 	(*ist)->timer_g_length = DEFAULT_T1;
@@ -57,7 +58,7 @@ __osip_ist_init (osip_ist_t ** ist, osip_t * osip, osip_message_t * invite)
 	(*ist)->timer_i_start.tv_sec = -1;	/* not started */
       }
     else
-      {				/* TCP is used: */
+      {				/* reliable protocol is used: */
 	(*ist)->timer_g_length = -1;	/* A is not ACTIVE */
 	(*ist)->timer_i_length = 0;	/* MUST do the transition immediatly */
 	(*ist)->timer_g_start.tv_sec = -1;	/* not started */
@@ -90,7 +91,7 @@ osip_event_t *
 __osip_ist_need_timer_g_event (osip_ist_t * ist, state_t state, int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ist == NULL)
     return NULL;
@@ -98,7 +99,7 @@ __osip_ist_need_timer_g_event (osip_ist_t * ist, state_t state, int transactioni
     {
       if (ist->timer_g_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ist->timer_g_start, >))
+      if (osip_timercmp (&now, &ist->timer_g_start, >))
 	return __osip_event_new (TIMEOUT_G, transactionid);
     }
   return NULL;
@@ -109,7 +110,7 @@ __osip_ist_need_timer_h_event (osip_ist_t * ist, state_t state,
 			       int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ist == NULL)
     return NULL;
@@ -118,7 +119,7 @@ __osip_ist_need_timer_h_event (osip_ist_t * ist, state_t state,
       /* may need timer H */
       if (ist->timer_h_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ist->timer_h_start, >))
+      if (osip_timercmp (&now, &ist->timer_h_start, >))
 	return __osip_event_new (TIMEOUT_H, transactionid);
     }
   return NULL;
@@ -129,7 +130,7 @@ __osip_ist_need_timer_i_event (osip_ist_t * ist, state_t state,
 			       int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (ist == NULL)
     return NULL;
@@ -138,7 +139,7 @@ __osip_ist_need_timer_i_event (osip_ist_t * ist, state_t state,
       /* may need timer I */
       if (ist->timer_i_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &ist->timer_i_start, >))
+      if (osip_timercmp (&now, &ist->timer_i_start, >))
 	return __osip_event_new (TIMEOUT_I, transactionid);
     }
   return NULL;
