@@ -52,17 +52,18 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip,
     if (proto == NULL)
       goto ii_error_1;
 
-    i = osip_strncasecmp (proto, "TCP", 3);
-    if (i != 0)
+    if (osip_strcasecmp (proto, "TCP") != 0
+	&& osip_strcasecmp (proto, "TLS") !=0
+	&& osip_strcasecmp (proto, "SCTP") !=0)
       {
 	(*nict)->timer_e_length = DEFAULT_T1;
 	(*nict)->timer_k_length = DEFAULT_T4;
-	gettimeofday (&(*nict)->timer_e_start, NULL);
+	osip_gettimeofday (&(*nict)->timer_e_start, NULL);
 	add_gettimeofday (&(*nict)->timer_e_start, (*nict)->timer_e_length);
 	(*nict)->timer_k_start.tv_sec = -1;	/* not started */
       }
     else
-      {				/* TCP is used: */
+      {				/* reliable protocol is used: */
 	(*nict)->timer_e_length = -1;	/* E is not ACTIVE */
 	(*nict)->timer_k_length = 0;	/* MUST do the transition immediatly */
 	(*nict)->timer_e_start.tv_sec = -1;
@@ -73,6 +74,17 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip,
   /* for PROXY, the destination MUST be set by the application layer,
      this one may not be correct. */
   osip_message_get_route (request, 0, &route);
+  if (route != NULL && route->url!=NULL)
+    {
+      osip_uri_param_t *lr_param;
+      osip_uri_uparam_get_byname(route->url, "lr", &lr_param);
+      if (lr_param==NULL)
+	{
+	  /* using uncompliant proxy: destination is the request-uri */
+	  route = NULL;
+	}
+    }
+
   if (route != NULL)
     {
       int port = 5060;
@@ -83,10 +95,18 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip,
 				 port);
     }
   else
-    (*nict)->port = 5060;
+    {
+      int port = 5060;
+      
+      if (request->req_uri->port != NULL)
+	port = osip_atoi (request->req_uri->port);
+      osip_nict_set_destination ((*nict),
+				 osip_strdup (request->req_uri->host),
+				 port);
+    }
 
   (*nict)->timer_f_length = 64 * DEFAULT_T1;
-  gettimeofday (&(*nict)->timer_f_start, NULL);
+  osip_gettimeofday (&(*nict)->timer_f_start, NULL);
   add_gettimeofday (&(*nict)->timer_f_start, (*nict)->timer_f_length);
 
   /* Oups! a Bug! */
@@ -129,7 +149,7 @@ osip_event_t *
 __osip_nict_need_timer_e_event (osip_nict_t * nict, state_t state, int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (nict == NULL)
     return NULL;
@@ -137,7 +157,7 @@ __osip_nict_need_timer_e_event (osip_nict_t * nict, state_t state, int transacti
     {
       if (nict->timer_e_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &nict->timer_e_start, >))
+      if (osip_timercmp (&now, &nict->timer_e_start, >))
 	return __osip_event_new (TIMEOUT_E, transactionid);
     }
   return NULL;
@@ -148,7 +168,7 @@ __osip_nict_need_timer_f_event (osip_nict_t * nict, state_t state,
 				int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (nict == NULL)
     return NULL;
@@ -156,7 +176,7 @@ __osip_nict_need_timer_f_event (osip_nict_t * nict, state_t state,
     {
       if (nict->timer_f_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &nict->timer_f_start, >))
+      if (osip_timercmp (&now, &nict->timer_f_start, >))
 	return __osip_event_new (TIMEOUT_F, transactionid);
     }
   return NULL;
@@ -167,7 +187,7 @@ __osip_nict_need_timer_k_event (osip_nict_t * nict, state_t state,
 				int transactionid)
 {
   struct timeval now;
-  gettimeofday (&now, NULL);
+  osip_gettimeofday (&now, NULL);
 
   if (nict == NULL)
     return NULL;
@@ -175,7 +195,7 @@ __osip_nict_need_timer_k_event (osip_nict_t * nict, state_t state,
     {
       if (nict->timer_k_start.tv_sec == -1)
 	return NULL;
-      if (timercmp (&now, &nict->timer_k_start, >))
+      if (osip_timercmp (&now, &nict->timer_k_start, >))
 	return __osip_event_new (TIMEOUT_K, transactionid);
     }
   return NULL;
