@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 
 #include <osipparser2/osip_port.h>
 #include <osipparser2/osip_message.h>
+#include <osipparser2/osip_parser.h>
 #include "parser.h"
 
 int
@@ -30,7 +31,7 @@ osip_message_set_allow (osip_message_t * sip, const char *hvalue)
   osip_allow_t *allow;
   int i;
 
-  if (hvalue==NULL || hvalue[0]=='\0')
+  if (hvalue == NULL || hvalue[0] == '\0')
     return 0;
 
   i = osip_allow_init (&allow);
@@ -42,15 +43,14 @@ osip_message_set_allow (osip_message_t * sip, const char *hvalue)
       osip_allow_free (allow);
       return -1;
     }
-#ifdef USE_TMP_BUFFER
   sip->message_property = 2;
-#endif
   osip_list_add (sip->allows, allow, -1);
   return 0;
 }
 
 int
-osip_message_get_allow (const osip_message_t * sip, int pos, osip_allow_t ** dest)
+osip_message_get_allow (const osip_message_t * sip, int pos,
+			osip_allow_t ** dest)
 {
   osip_allow_t *allow;
 
