@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -155,8 +155,11 @@ osip_from_parse (osip_from_t * from, const char *hvalue)
      if the fisrt quote is after '<' then
      this is not a quote for a displayname.
    */
-  if (displayname > url)
-    displayname = NULL;
+  if (displayname!=NULL)
+    {
+      if (displayname > url)
+	displayname = NULL;
+    }
 
   if ((displayname == NULL) && (url != NULL))
     {				/* displayname IS A '*token' (not a quoted-string) */
@@ -315,7 +318,7 @@ osip_from_to_str (const osip_from_t * from, char **dest)
 	else
 	  plen = strlen (u_param->gname) + strlen (u_param->gvalue) + 3;
 	len = len + plen;
-	buf = (char *) realloc (buf, len);
+	buf = (char *) osip_realloc (buf, len);
 	tmp = buf;
 	tmp = tmp + strlen (tmp);
 	if (u_param->gvalue == NULL)
