@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -212,6 +212,9 @@ osip_header_to_str (const osip_header_t * header, char **dest)
     sprintf (*dest, "%s: %s", header->hname, header->hvalue);
   else
     sprintf (*dest, "%s: ", header->hname);
+
+  if (*dest[0] > 'a' && *dest[0] < 'z')
+    *dest[0] = (*dest[0]-32);
   return 0;
 }
 
