@@ -26,7 +26,9 @@
 int
 osip_list_init (osip_list_t * li)
 {
-  li->nb_elt = 0;
+  if (li==NULL)
+    return -1;
+  memset(li, 0, sizeof(osip_list_t));
   return 0;			/* ok */
 }
 
@@ -42,7 +44,8 @@ osip_list_special_free (osip_list_t * li, void *(*free_func) (void *))
     {
       element = (void *) osip_list_get (li, pos);
       osip_list_remove (li, pos);
-      free_func (element);
+      if (free_func!=NULL)
+	free_func (element);
     }
   osip_free (li);
 }
@@ -96,31 +99,41 @@ osip_list_add (osip_list_t * li, void *el, int pos)
   __node_t *ntmp;
   int i = 0;
 
-  if (pos == -1 || pos >= li->nb_elt)
-    {				/* insert at the end  */
-      pos = li->nb_elt;
-    }
+  if (li == NULL) return -1;
 
   if (li->nb_elt == 0)
     {
 
       li->node = (__node_t *) osip_malloc (sizeof (__node_t));
+      if (li->node == NULL) return -1;
       li->node->element = el;
+      li->node->next = NULL;
       li->nb_elt++;
       return li->nb_elt;
     }
 
+  if (pos == -1 || pos >= li->nb_elt)
+    {				/* insert at the end  */
+      pos = li->nb_elt;
+    }
+
   ntmp = li->node;		/* exist because nb_elt>0  */
 
-  if (pos == 0)
+  if (pos == 0) /* pos = 0 insert before first elt  */
     {
       li->node = (__node_t *) osip_malloc (sizeof (__node_t));
+      if (li->node == NULL)
+	{
+	  /* leave the list unchanged */
+	  li->node=ntmp;
+	  return -1;
+	}
       li->node->element = el;
       li->node->next = ntmp;
       li->nb_elt++;
       return li->nb_elt;
     }
-  /* pos = 0 insert before first elt  */
+  
 
   while (pos > i + 1)
     {
@@ -133,8 +146,10 @@ osip_list_add (osip_list_t * li, void *el, int pos)
   if (pos == li->nb_elt)
     {
       ntmp->next = (__node_t *) osip_malloc (sizeof (__node_t));
+      if (ntmp->next == NULL) return -1; /* leave the list unchanged */
       ntmp = (__node_t *) ntmp->next;
       ntmp->element = el;
+      ntmp->next = NULL;
       li->nb_elt++;
       return li->nb_elt;
     }
@@ -144,6 +159,11 @@ osip_list_add (osip_list_t * li, void *el, int pos)
     __node_t *nextnode = (__node_t *) ntmp->next;
 
     ntmp->next = (__node_t *) osip_malloc (sizeof (__node_t));
+    if (ntmp->next ==  NULL) {
+      /* leave the list unchanged */
+      ntmp->next=nextnode;
+      return -1;
+    }
     ntmp = (__node_t *) ntmp->next;
     ntmp->element = el;
     ntmp->next = nextnode;
@@ -159,9 +179,11 @@ osip_list_get (const osip_list_t * li, int pos)
   __node_t *ntmp;
   int i = 0;
 
+  if (li == NULL) return NULL;
+
   if (pos < 0 || pos >= li->nb_elt)
     /* element does not exist */
-    return 0;
+    return NULL;
 
 
   ntmp = li->node;		/* exist because nb_elt>0 */
@@ -174,6 +196,56 @@ osip_list_get (const osip_list_t * li, int pos)
   return ntmp->element;
 }
 
+/* added by bennewit@cs.tu-berlin.de */
+void * osip_list_get_first( osip_list_t * li, osip_list_iterator_t * iterator )
+{
+  if ( 0 >= li->nb_elt) {
+    iterator->actual = 0;
+    return 0;
+  }
+
+  iterator->actual = li->node;
+  iterator->prev = &li->node;
+  iterator->li = li;
+  iterator->pos = 0;
+
+  return li->node->element;
+}
+
+/* added by bennewit@cs.tu-berlin.de */
+void * osip_list_get_next( osip_list_iterator_t * iterator )
+{
+  iterator->prev = (__node_t**)&(iterator->actual->next);
+  iterator->actual = iterator->actual->next;
+  ++(iterator->pos);
+
+  if ( osip_list_iterator_has_elem( *iterator ) ) {
+    return iterator->actual->element;
+  }
+
+  iterator->actual = 0;
+  return 0;
+}
+
+/* added by bennewit@cs.tu-berlin.de */
+void * osip_list_iterator_remove( osip_list_iterator_t * iterator )
+{
+  if ( osip_list_iterator_has_elem( *iterator ) ) {
+    --(iterator->li->nb_elt);
+
+    *(iterator->prev) = iterator->actual->next;
+
+    osip_free( iterator->actual );
+    iterator->actual = *(iterator->prev);
+  }
+  
+  if ( osip_list_iterator_has_elem( *iterator ) ) {
+    return iterator->actual->element;
+  }
+
+  return 0;
+}
+
 /* return -1 if failed */
 int
 osip_list_remove (osip_list_t * li, int pos)
@@ -182,6 +254,8 @@ osip_list_remove (osip_list_t * li, int pos)
   __node_t *ntmp;
   int i = 0;
 
+  if (li == NULL) return -1;
+
   if (pos < 0 || pos >= li->nb_elt)
     /* element does not exist */
     return -1;
