@@ -27,8 +27,6 @@
 static void osip_util_replace_all_lws (char *sip_message);
 static int osip_message_set__header (osip_message_t * sip, const char *hname,
 				     const char *hvalue);
-static int msg_handle_multiple_values (osip_message_t * sip, char *hname,
-				       char *hvalue);
 static int msg_headers_parse (osip_message_t * sip,
 			      const char *start_of_header, const char **body);
 static int msg_osip_body_parse (osip_message_t * sip,
@@ -180,6 +178,9 @@ __osip_message_startline_parseresp (osip_message_t * dest, const char *buf,
       return -1;
     }
 
+  if (dest->status_code==0)
+    return -1;
+
   {
     const char *hp = reasonphrase;
 
@@ -220,16 +221,34 @@ __osip_message_startline_parse (osip_message_t * dest, const char *buf,
 
 int
 __osip_find_next_occurence (const char *str, const char *buf,
-			    const char **index_of_str)
+			    const char **index_of_str, const char *end_of_buf)
 {
+  int i;
   *index_of_str = NULL;		/* AMD fix */
   if ((NULL == str) || (NULL == buf))
     return -1;
   /* TODO? we may prefer strcasestr instead of strstr? */
-  *index_of_str = strstr (buf, str);
-  if (NULL == (*index_of_str))
-    return -1;
-  return 0;
+  for (i=0;i<1000;i++)
+    {
+      *index_of_str = strstr (buf, str);
+      if (NULL == (*index_of_str))
+	{
+	  /* if '\0' (when binary data is used) is located before the separator,
+	     then we have to continue searching */
+	  const char *ptr = buf + strlen(buf);
+	  if (end_of_buf-ptr>0)
+	    {
+	      buf = ptr+1;
+	      continue;
+	    }
+	  return -1;
+	}
+      return 0;
+    }
+  OSIP_TRACE (osip_trace
+	      (__FILE__, __LINE__, OSIP_BUG, NULL,
+	       "This was probably an infinite loop?\n"));
+  return -1;
 }
 
 /* This method replace all LWS with SP located before the
@@ -408,8 +427,9 @@ osip_message_set__header (osip_message_t * sip, const char *hname,
   return 0;
 }
 
-static int
-msg_handle_multiple_values (osip_message_t * sip, char *hname, char *hvalue)
+int
+osip_message_set_multiple_header (osip_message_t * sip, char *hname,
+				  char *hvalue)
 {
   int i;
   char *ptr;			/* current location of the search */
@@ -418,7 +438,7 @@ msg_handle_multiple_values (osip_message_t * sip, char *hname, char *hvalue)
   char *end;			/* end of a header */
   char *quote1;			/* first quote of a pair of quotes   */
   char *quote2;			/* second quuote of a pair of quotes */
-  int   hname_len;
+  size_t   hname_len;
 
   beg = hvalue;
   end = NULL;
@@ -614,7 +634,7 @@ msg_headers_parse (osip_message_t * sip, const char *start_of_header,
       /* are separated by commas. But, a comma may be part of a   */
       /* quoted-string ("here, and there" is an example where the */
       /* comma is not a separator!) */
-      i = msg_handle_multiple_values (sip, hname, hvalue);
+      i = osip_message_set_multiple_header (sip, hname, hvalue);
 
       osip_free (hname);
       if (hvalue!=NULL) osip_free (hvalue);
@@ -650,6 +670,7 @@ msg_osip_body_parse (osip_message_t * sip, const char *start_of_buf,
 {
   const char *start_of_body;
   const char *end_of_body;
+  const char *end_of_buf;
   char *tmp;
   int i;
 
@@ -744,11 +765,14 @@ msg_osip_body_parse (osip_message_t * sip, const char *start_of_buf,
 
   *next_body = NULL;
   start_of_body = start_of_buf;
+
+  end_of_buf = start_of_buf+length;
+
   for (;;)
     {
       i =
 	__osip_find_next_occurence (sep_boundary, start_of_body,
-				    &start_of_body);
+				    &start_of_body, end_of_buf);
       if (i == -1)
 	{
 	  osip_free (sep_boundary);
@@ -757,7 +781,7 @@ msg_osip_body_parse (osip_message_t * sip, const char *start_of_buf,
       i =
 	__osip_find_next_occurence (sep_boundary,
 				    start_of_body + strlen (sep_boundary),
-				    &end_of_body);
+				    &end_of_body, end_of_buf);
       if (i == -1)
 	{
 	  osip_free (sep_boundary);
@@ -771,7 +795,10 @@ msg_osip_body_parse (osip_message_t * sip, const char *start_of_buf,
 
       tmp = osip_malloc (end_of_body - start_of_body + 2);
       if (tmp == NULL)
-	return -1;
+	{
+	  osip_free (sep_boundary);
+	  return -1;
+	}
       memcpy (tmp, start_of_body, end_of_body - start_of_body);
       tmp[end_of_body - start_of_body]='\0';
       
@@ -893,7 +920,7 @@ osip_message_fix_last_via_header (osip_message_t * request,
       if (rport->gvalue == NULL)
 	{
 	  rport->gvalue = (char *) osip_malloc (9);
-#if (defined WIN32 || defined _WIN32_WCE)
+#if !defined __PALMOS__ && (defined WIN32 || defined _WIN32_WCE)   
 	  _snprintf (rport->gvalue, 8, "%i", port);
 #else
 	  snprintf (rport->gvalue, 8, "%i", port);
