@@ -42,10 +42,28 @@
 #include <ctype.h>
 #endif
 
-#include <time.h>
+#if defined(__PALMOS__) && (__PALMOS__ < 0x06000000)
+#	include <TimeMgr.h>
+#	include <SysUtils.h>
+#	include <SystemMgr.h>
+#	include <StringMgr.h>
+#else
+#	include <time.h>
+#endif
 
 #if defined(__VXWORKS_OS__)
 #include <selectLib.h>
+
+/* needed for snprintf replacement */
+#include <vxWorks.h>
+#include <fioLib.h>
+#include <string.h>
+
+#elif defined(__PALMOS__)
+#	if __PALMOS__ >= 0x06000000
+#		include <sys/time.h>
+#		include <SysThread.h>
+#	endif
 #elif (!defined(WIN32) && !defined(_WIN32_WCE))
 #include <sys/time.h>
 #elif defined(WIN32)
@@ -100,12 +118,25 @@ osip_fallback_random_number ()
   if (!random_seed_set)
     {
       unsigned int ticks;
-#ifdef WIN32
+#ifdef __PALMOS__
+#	if __PALMOS__ < 0x06000000
+      SysRandom((Int32)TimGetTicks());
+#	else
+      struct timeval tv;
+      gettimeofday (&tv, NULL);
+      srand (tv.tv_usec);
+      ticks = tv.tv_sec + tv.tv_usec;
+#	endif
+#elif defined(WIN32)
       LARGE_INTEGER lCount;
       QueryPerformanceCounter(&lCount);
       ticks = lCount.LowPart + lCount.HighPart;
 #elif defined(_WIN32_WCE)
       ticks = GetTickCount();
+#elif defined(__VXWORKS_OS__)
+      struct timespec tp;
+      clock_gettime(CLOCK_REALTIME, &tp);
+      ticks = tp.tv_sec+tp.tv_nsec;
 #else
       struct timeval tv;
       int fd;
@@ -189,11 +220,7 @@ osip_atoi (const char *number)
 char *
 osip_strncpy (char *dest, const char *src, size_t length)
 {
-#ifdef WIN32
   strncpy (dest, src, length);
-#else
-  strncpy (dest, src, length);
-#endif
   dest[length] = '\0';
   return dest;
 }
@@ -222,7 +249,16 @@ __osip_sdp_append_string (char *string, size_t size, char *cur,
 void
 osip_usleep (int useconds)
 {
-#ifdef WIN32
+#if defined(__PALMOS__) && (__PALMOS__ >= 0x06000000)
+	/* This bit will work for the Protein API, but not the Palm 68K API */
+	nsecs_t nanoseconds = useconds*1000;
+	SysThreadDelay(nanoseconds, P_ABSOLUTE_TIMEOUT);
+#elif defined(__PALMOS__) && (__PALMOS__ < 0x06000000)
+	UInt32 stoptime = TimGetTicks() + (useconds / 1000000) * SysTicksPerSecond();
+	while (stoptime > TimGetTicks())
+		/* I wish there was some type of yield function here */
+		;	
+#elif defined(WIN32) 
   Sleep (useconds / 1000);
 #else
   struct timeval delay;
@@ -296,7 +332,16 @@ osip_tolower (char *word)
 int
 osip_strcasecmp (const char *s1, const char *s2)
 {
-#if (!defined WIN32 && !defined _WIN32_WCE)
+#if defined(__VXWORKS_OS__)
+  while ( (*s1 != '\0') && (tolower(*s1) == tolower(*s2)) )
+  {
+    s1++;
+    s2++;
+  }
+  return (tolower(*s1) - tolower(*s2));
+#elif defined(__PALMOS__) && (__PALMOS__ < 0x06000000)
+  return StrCaselessCompare(s1, s2);
+#elif defined(__PALMOS__) || (!defined WIN32 && !defined _WIN32_WCE)
   return strcasecmp (s1, s2);
 #else
   return _stricmp (s1, s2);
@@ -306,7 +351,20 @@ osip_strcasecmp (const char *s1, const char *s2)
 int
 osip_strncasecmp (const char *s1, const char *s2, size_t len)
 {
-#if (!defined WIN32 && !defined _WIN32_WCE)
+#if defined(__VXWORKS_OS__)
+  if ( len == 0 ) return 0;
+  while ( (len > 0) && (tolower(*s1) == tolower(*s2)) )
+  {
+    len--;
+    if ( (len == 0) || (*s1 == '\0') || (*s2 == '\0') )
+            break;
+    s1++;
+    s2++;
+  }
+  return tolower(*s1) - tolower(*s2);
+#elif defined(__PALMOS__) && (__PALMOS__ < 0x06000000)
+  return StrNCaselessCompare(s1, s2, len);
+#elif defined(__PALMOS__) || (!defined WIN32 && !defined _WIN32_WCE)
   return strncasecmp (s1, s2, len);
 #else
   return _strnicmp (s1, s2, len);
@@ -684,8 +742,8 @@ int
 osip_trace (char *fi, int li, osip_trace_level_t level, FILE * f, char *chfr,
 	    ...)
 {
-  va_list ap;
 #ifdef ENABLE_TRACE
+  va_list ap;
 
 #if !defined(WIN32) && !defined(SYSTEM_LOGGER_ENABLED)
   if (logfile == NULL && use_syslog == 0 && trace_func == NULL)
@@ -848,3 +906,61 @@ void osip_set_allocators(osip_malloc_func_t  *malloc_func,
 
 #endif
 
+#if defined(__VXWORKS_OS__)
+
+typedef struct
+{
+  char* str;
+  int   max;
+  int   len;
+} _context;
+
+STATUS _cb_snprintf( char* buffer, int nc, int arg );
+
+STATUS _cb_snprintf( char* buffer, int nc, int arg )
+{
+  _context *ctx = (_context*)arg;
+  
+  if( ctx->max - ctx->len - nc < 1 ) /* retain 1 pos for terminating \0 */
+  {
+    nc = ctx->max - ctx->len - 1;
+  }
+ 
+  if( nc > 0 )
+  {
+    memcpy( ctx->str + ctx->len, buffer, nc );
+    ctx->len += nc;
+  }
+
+  ctx->str[ctx->len] = '\0';
+
+  return OK;
+}
+
+
+int osip_vsnprintf( char* buf, int max, const char *fmt, va_list ap )
+{
+  _context ctx;  
+  ctx.str = buf;
+  ctx.max = max;
+  ctx.len = 0;
+
+  if( fioFormatV( fmt, ap, _cb_snprintf, (int)&ctx ) != OK )
+  {
+    return -1;
+  }
+
+  return ctx.len;
+}
+
+int osip_snprintf( char* buf, int max, const char* fmt, ... )
+{
+  int retval;
+  va_list ap;
+  va_start( ap, fmt );
+  retval = osip_vsnprintf( buf, max, fmt, ap );
+  va_end( ap );
+  return retval;
+}
+
+#endif
