@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -600,6 +600,65 @@ sdp_message_a_attribute_del (sdp_message_t * sdp, int pos_media, char *att_field
     }
   return 0;
 }
+
+int
+sdp_message_a_attribute_del_at_index (sdp_message_t * sdp, int pos_media, char *att_field, int pos_attr)
+{
+  int i;
+  sdp_media_t *med;
+  sdp_attribute_t* attr;
+  
+  if (sdp == NULL)
+    return -1;
+  if ((pos_media != -1) && (osip_list_size (sdp->m_medias) < pos_media + 1))
+    return -1;
+  if (pos_media==-1)
+    {
+      if (pos_attr == -1)
+	{
+	  for (i=0;i<osip_list_size(sdp->a_attributes);)
+	    {
+	      attr = osip_list_get(sdp->a_attributes,i);
+	      if (strcmp(attr->a_att_field,att_field)==0)
+		{
+		  osip_list_remove(sdp->a_attributes,i);
+		  sdp_attribute_free(attr);
+		}
+	      else
+		i++;
+	    }
+        }
+      else if ((attr = osip_list_get(sdp->a_attributes,pos_attr)) != NULL)
+	{
+	  osip_list_remove(sdp->a_attributes,pos_attr);
+	  sdp_attribute_free(attr);
+	}
+      return 0;
+    }
+  med = (sdp_media_t *) osip_list_get (sdp->m_medias, pos_media);
+  if (med == NULL)
+    return -1;
+  for (i=0;i<osip_list_size(med->a_attributes);)
+    {
+      if (pos_attr == -1)
+	{
+	  attr = osip_list_get(med->a_attributes,i);
+	  if (strcmp(attr->a_att_field,att_field)==0)
+	    {
+	      osip_list_remove(med->a_attributes,i);
+	      sdp_attribute_free(attr);
+	    }
+	  else
+	    i++;
+	}
+      else if ((attr = osip_list_get(med->a_attributes,pos_attr)) != NULL)
+	{
+	  osip_list_remove(med->a_attributes,pos_attr);
+	  sdp_attribute_free(attr);
+	}
+    }
+  return 0;
+}
  
 
 sdp_attribute_t *
@@ -697,6 +756,19 @@ sdp_message_m_number_of_port_get (sdp_message_t * sdp, int pos_media)
   return med->m_number_of_port;
 }
 
+int
+sdp_message_m_port_set (sdp_message_t * sdp, int pos_media, char *port)
+{
+  sdp_media_t *med = osip_list_get (sdp->m_medias, pos_media);
+  
+  if (med == NULL)
+    return -1;
+  if (med->m_port)
+    osip_free(med->m_port);
+  med->m_port = port;
+  return 0;
+}
+
 char *
 sdp_message_m_proto_get (sdp_message_t * sdp, int pos_media)
 {
@@ -727,3 +799,18 @@ sdp_message_m_payload_get (sdp_message_t * sdp, int pos_media, int pos)
     return NULL;
   return (char *) osip_list_get (med->m_payloads, pos);
 }
+
+int
+sdp_message_m_payload_del (sdp_message_t * sdp, int pos_media, int pos)
+{
+  sdp_media_t *med = osip_list_get (sdp->m_medias, pos_media);
+  char *payload;
+  
+  if (med == NULL)
+    return -1;
+  if ((payload = osip_list_get (med->m_payloads, pos)) == NULL)
+    return -1;
+  osip_list_remove(med->m_payloads, pos);
+  osip_free(payload);
+  return 0;
+}
